Package: Rook
Title: HTTP Web Server for R
Version: 1.2
Date: 2022-11-05
Description: An HTTP web server for R with a documented API to interface between R and the server. The documentation contains the Rook specification and details for building and running Rook applications. To get started, be sure and read the 'Rook' help file first.
Encoding: UTF-8
Depends: R (>= 2.13.0)
Imports: utils, tools, methods, brew
License: GPL-2
LazyLoad: yes
URL: https://github.com/evanbiederstedt/rook
BugReports: https://github.com/evanbiederstedt/rook/issues
Author: Jeffrey Horner [aut], Evan Biederstedt [aut, cre]
Maintainer: Evan Biederstedt <evan.biederstedt@gmail.com>
NeedsCompilation: yes
Packaged: 2022-11-05 23:55:46 UTC; evanbiederstedt
Repository: CRAN
Date/Publication: 2022-11-07 08:50:19 UTC
Built: R 4.4.0; amd64-portbld-midnightbsd4.0.2; 2026-02-18 02:42:31 UTC; unix
