Package: Rook
Version: 1.2.1
Title: HTTP Web Server for R
Authors@R: c(person(given = "Jeffrey",
                    family = "Horner",
                    role = "aut"),
             person(given = "Evan",
                    family = "Biederstedt",
                    role = c("aut", "cre"),
                    email = "evan.biederstedt@gmail.com"))
Date: 2026-06-30
Description: An HTTP web server for R with a documented API to interface between R and the server. The documentation contains the Rook specification and details for building and running Rook applications. To get started, be sure and read the 'Rook' help file first.
Encoding: UTF-8
Depends: R (>= 2.13.0)
Imports: utils, tools, methods, brew
License: GPL-2
LazyLoad: yes
URL: https://github.com/evanbiederstedt/rook
BugReports: https://github.com/evanbiederstedt/rook/issues
Maintainer: Evan Biederstedt <evan.biederstedt@gmail.com>
NeedsCompilation: yes
Packaged: 2026-06-30 19:30:21 UTC; evanbiederstedt
Author: Jeffrey Horner [aut],
  Evan Biederstedt [aut, cre]
Repository: CRAN
Date/Publication: 2026-07-01 00:00:02 UTC
