\name{suspend_console}
\alias{suspend_console}
\title{
Suspend the R console 
}
\description{
Calls Sys.sleep in a never-ending while loop to mimic suspension of the R console.
}
\usage{
suspend_console()
}
\value{
    No value is ever returned.
}
\seealso{
\code{\link{Rook}}.
}
\keyword{function}
