\name{RhttpdInputStream-class}
\Rdversion{1.1}
\docType{class}
\alias{RhttpdInputStream-class}
\alias{RhttpdInputStream}

\title{Class \code{RhttpdInputStream}}
\description{
An internal class used by \code{\link{Rhttpd}}.
}
\examples{
showClass("RhttpdInputStream")
}
\keyword{classes}
