\name{App-class}
\Rdversion{1.1}
\docType{class}
\alias{App-class}
\alias{App}

\title{Class \code{App}}
\description{
Abstract class from which \code{Middleware} and \code{Builder} inherit. Provides the \code{app} field.

\code{App} can also be used to instantiate reference classed applications wrapped around a function. See \code{\link{Middleware}} for an example.
}
\section{Fields}{
  \describe{
    \item{\code{app}:}{A Rook application.}
  }
}
\section{Methods}{
  \describe{
    \item{\code{new(app=NULL)}:}{ Creates a new \code{App} object. \code{app} is any \code{Rook} aware R object.}
  }
}
\seealso{
\code{\link{is_rookable}}, \code{\link{Builder}}, and \code{\link{Middleware}}.
}
\keyword{classes}
